#import "FolderEntity.h"

#import "AppDelegate.h"

@implementation FolderEntity

@synthesize name = _name;
@synthesize position = _position;
@synthesize children = _children;
@synthesize recordIdentifier = _recordIdentifier;
@synthesize parent = _parentIdentifier;
@synthesize updated;

+ (NSString*)entityName;
{
	return @"com.apple.bookmarks.Folder";
}

- (id)init
{
  if (![super init]) return nil;
  
  [self addObserver:self forKeyPath:@"name" options:NSKeyValueObservingOptionNew context:nil];
  [self addObserver:self forKeyPath:@"position" options:NSKeyValueObservingOptionNew context:nil];
  [self addObserver:self forKeyPath:@"children" options:NSKeyValueObservingOptionNew context:nil];
  [self addObserver:self forKeyPath:@"parent" options:NSKeyValueObservingOptionNew context:nil];
  
  return self;
}

- (void)observeValueForKeyPath:(NSString*)keyPath 
                      ofObject:(id)object 
                        change:(NSDictionary*)change 
                       context:(void*)context
{
  [self setUpdated:YES];
}

- (void)dealloc
{
  [self removeObserver:self forKeyPath:@"name"];
  [self removeObserver:self forKeyPath:@"position"];
  [self removeObserver:self forKeyPath:@"children"];
  [self removeObserver:self forKeyPath:@"parent"];
  [_name release], _name = nil;
  [_children release], _children = nil;
  [_recordIdentifier release], _recordIdentifier = nil;
  [_parentIdentifier release], _parentIdentifier = nil;
  [super dealloc];
}  
- (int)recordCount;
{
  return [_children count];
}

- (BOOL)isLeaf;
{
  return NO;
}

- (NSDictionary*)fullRecord;
{
  NSMutableDictionary *dict = [NSMutableDictionary dictionary];
  [dict setValue:[FolderEntity entityName] forKey:ISyncRecordEntityNameKey];
  [dict setValue:[self name] forKey:@"name"];
  [dict setValue:[NSNumber numberWithInt:[self position]] forKey:@"position"];
  [dict setValue:[self children] forKey:@"children"];
  [dict setValue:[self parent] forKey:@"parent"];
  return dict;
}

- (NSArray*)childRecords;
{
  NSMutableArray *records = [NSMutableArray array];
  for (NSString *ident in _children) {
    [records addObject:[[NSApp delegate] objectForRecordIdentifier:ident]];
  }
  return records;
}

- (void)setChildren:(NSArray*)children
{
  NSMutableArray *temp = [children mutableCopy];
  [_children release];
  _children = temp;
}

- (FolderEntity*)parentFolder;
{
  return [[NSApp delegate] objectForRecordIdentifier:[self parent]];
}

- (NSString*)secondColumn;
{
  return [NSString stringWithFormat:@"Items: %u", [_children count]];
}

@end
